from .  import BaseResource, User


class Release(BaseResource):
    _strs = ['description', 'id', 'user', 'commit', 'addons']
    _ints = ['version']
    _dates = ['created_at', 'updated_at']
    _map = {'user': User}
    _pks = ['id', 'version']
    order_by = 'version'

    def __init__(self):
        self.app = None
        super(Release, self).__init__()

    def __repr__(self):
        return "<release '{0} {1} {2}'>\n".format(self.version, self.created_at, self.description)



from ..models import BaseResource, User, Stack
from ..rendezvous import Rendezvous
from ..structures import DynoListResource

from .addon import Addon
from .collaborator import Collaborator
from .configvars import ConfigVars
from .domain import Domain
from .dyno import Dyno
from .formation import Formation
from .logdrain import LogDrain
from .logsession import LogSession
from .region import Region
from .release import Release

from pprint import pprint # NOQA
import sys

if sys.version_info > (3, 0):
    from urllib.parse import quote
else:
    from urllib import quote # noqa


class App(BaseResource):
    """Heroku App."""

    _strs = ['buildpack_provided_description', 'git_url', 'id', 'name', 'web_url']
    _ints = ['slug_size', 'repo_size']
    _bools = ['maintenance']
    _dates = ['archived_at', 'created_at', 'released_at', 'updated_at']
    _map = {'region': Region, 'owner': User, 'stack': Stack}
    _pks = ['name', 'id']

    def __init__(self):
        super(App, self).__init__()

    def __repr__(self):
        return "<app '{0} - {1}'>".format(self.name, self.id)

    def addons(self, **kwargs):
        """
        Returns a list of your apps as app objects.
        """
        return self._h._get_resources(
            resource=('apps', self.name, 'addons'),
            obj=Addon, app=self, **kwargs
        )

    def delete(self):
        """
        Destroys the current app
        """
        r = self._h._http_resource(
            method='DELETE',
            resource=('apps', self.id)
        )
        r.raise_for_status()
        return r.ok

    def add_collaborator(self, user_id_or_email, silent=False):
        """
        Adds a collaborator to your app
        [silent 1|0]  Specifies whether to email the collaborator or not
        """

        if silent:
            silent = True

        #commented out until api is fixed
        payload = {'silent': silent, 'user': user_id_or_email}

        r = self._h._http_resource(
            method='POST',
            resource=('apps', self.name, 'collaborators'),
            data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return Collaborator.new_from_dict(item, h=self._h, app=self)

    def remove_collaborator(self, id_or_email):
        """
        Removes a collaborator from a project
        options = id_or_email
        """
        r = self._h._http_resource(
            method='DELETE',
            resource=('apps', self.name, 'collaborators', id_or_email)
        )
        r.raise_for_status()

        return r.ok

    def install_addon(self, plan_id_or_name, config=None):

        payload = {}
        if not config:
            config = {}

        payload['plan'] = plan_id_or_name
        payload['config'] = config

        r = self._h._http_resource(
            method='POST',
            resource=('apps', self.name, 'addons'),
            data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return Addon.new_from_dict(item, h=self._h, app=self)

    def remove_addon(self, id):
        r = self._h._http_resource(
            method='DELETE',
            resource=('apps', self.id, 'addons', id),
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return Addon.new_from_dict(item, h=self._h, app=self)

    def collaborators(self, **kwargs):
        """The collaborators for this app."""
        return self._h._get_resources(
            resource=('apps', self.name, 'collaborators'),
            obj=Collaborator, app=self, **kwargs
        )

    def config(self):
        """The envs for this app."""

        return self._h._get_resource(
            resource=('apps', self.name, 'config-vars'),
            obj=ConfigVars, app=self
        )

    def update_config(self, config):
        payload = self._h._resource_serialize(config)
        r = self._h._http_resource(
            method='PATCH',
            resource=('apps', self.id, 'config-vars'),
            data=payload
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return ConfigVars.new_from_dict(item, h=self._h, app=self)

    def domains(self, **kwargs):
        """The domains for this app."""
        return self._h._get_resources(
            resource=('apps', self.name, 'domains'),
            obj=Domain, app=self, **kwargs
        )

    def add_domain(self, hostname):

        r = self._h._http_resource(
            method='POST',
            resource=('apps', self.name, 'domains'),
            data=self._h._resource_serialize({'hostname': hostname})
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return Domain.new_from_dict(item, h=self._h, app=self)

    def remove_domain(self, hostname):
        r = self._h._http_resource(
            method='DELETE',
            resource=('apps', self.name, 'domains', hostname)
        )

        r.raise_for_status()

        return r.ok

    def dynos(self, **kwargs):
        """The proccesses for this app."""
        return self._h._get_resources(
            resource=('apps', self.name, 'dynos'),
            obj=Dyno, app=self, map=DynoListResource, **kwargs
        )

    def kill_dyno(self, dyno_id_or_name):
        r = self._h._http_resource(
            method='DELETE',
            resource=('apps', self.id, 'dynos', quote(dyno_id_or_name))
        )

        r.raise_for_status()

        return r.ok

    def restart(self):
        for dyno in self.dynos():
            if dyno.type != 'run':
                dyno.kill()
        return self

    def run_command_detached(self, command, size=1, env=None):
        """Run a remote command but do not wait for the command to complete"""
        return self.run_command(command, attach=False, printout=False, size=size, env=env)

    def run_command(self, command, attach=True, printout=True, size=1, env=None):
        """Run a remote command attach=True if you want to capture the output"""
        if attach:
            attach = True

        payload = {'command': command, 'attach': attach, 'size': size}
        if env:
            payload['env'] = env

        r = self._h._http_resource(
            method='POST',
            resource=('apps', self.name, 'dynos'),
            data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        dyno = Dyno.new_from_dict(item, h=self._h, app=self)

        if attach:
            output = Rendezvous(dyno.attach_url, printout).start()
            return output, dyno
        else:
            return dyno

    def process_formation(self, **kwargs):
        """The formation processes for this app."""
        return self._h._get_resources(
            resource=('apps', self.name, 'formation'),
            obj=Formation, app=self, **kwargs
        )

    def scale_formation_process(self, formation_id_or_name, quantity):
        assert(quantity == 0 or quantity)
        payload = {}
        payload['quantity'] = quantity

        r = self._h._http_resource(
            method='PATCH',
            resource=('apps', self.id, 'formation', formation_id_or_name),
            data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        return self._h._process_item(self._h._resource_deserialize(r.content.decode("utf-8")), Formation)

    def resize_formation_process(self, formation_id_or_name, size):
        assert(size == 0 or size)
        payload = {}
        payload['size'] = size

        r = self._h._http_resource(
            method='PATCH',
            resource=('apps', self.id, 'formation', formation_id_or_name),
            data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        return self._h._process_items(self._h._resource_deserialize(r.content.decode("utf-8")), Formation)

    @property
    def info(self):
        """Returns current info for this app."""

        return self._h._get_resource(
            resource=('apps', self.name),
            obj=App,
        )

    def labs(self, **kwargs):
        return self.features(**kwargs)

    def features(self, **kwargs):
        return self._h._get_resources(
            resource=('apps', self.id, 'features'),
            obj=AppFeature, app=self, **kwargs
        )

    def disable_feature(self, id_or_name):
        return self.update_feature(id_or_name, 0)

    def enable_feature(self, id_or_name):
        return self.update_feature(id_or_name, True)

    def update_feature(self, id_or_name, enabled):

        payload = {'enabled': enabled}
        r = self._h._http_resource(
            method='PATCH',
            resource=('apps', self.id, 'features', id_or_name),
            data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return AppFeature.new_from_dict(item, h=self._h, app=self)

    def logdrains(self, **kwargs):
        return self._h._get_resources(
            resource=('apps', self.id, 'log-drains'),
            obj=LogDrain, app=self, **kwargs
        )

    def create_logdrain(self, url):

        r = self._h._http_resource(
            method='POST',
            resource=('apps', self.id, 'log-drains'),
            data=self._h._resource_serialize({'url': url})
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return LogDrain.new_from_dict(item, h=self._h, app=self)

    def remove_logdrain(self, id_or_url):

        r = self._h._http_resource(
            method='DELETE',
            resource=('apps', self.id, 'log-drains', id_or_url),
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return LogDrain.new_from_dict(item, h=self._h, app=self)

    def rename(self, name):
        """Renames app to given name."""

        return self.update(name=name)

    def transfers(self, **kwargs):
        return self._h._get_resources(
            resource=('account', 'app-transfers'),
            obj=AppTransfer, app=self, **kwargs
        )

    def create_transfer(self, recipient_id_or_name):
        """Transfers app to given username's account."""

        payload = {'app': self.id, 'recipient': recipient_id_or_name}

        r = self._h._http_resource(
            method='PUT',
            resource=('account', 'app-transfers'),
            data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return AppTransfer.new_from_dict(item, h=self._h, app=self)

    def delete_transfer(self, id):
        r = self._h._http_resource(
            method='DELETE',
            resource=('account', 'app-transfers', id),
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return AppTransfer.new_from_dict(item, h=self._h, app=self)

    def enable_maintenance_mode(self):
        """Enables maintenance mode."""
        return self.update(maintenance=True)

    def disable_maintenance_mode(self):
        """Disables maintenance mode."""
        return self.update(maintenance=0)

    def update(self, maintenance=False, name=None):

        assert(maintenance or name or maintenance == 0)

        payload = {}
        if name:
            payload['name'] = name
        else:
            if maintenance or maintenance == 0:
                payload['maintenance'] = maintenance

        r = self._h._http_resource(
            method='PATCH',
            resource=('apps', self.id),
            data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return App.new_from_dict(item, h=self._h, app=self)

    def destroy(self):
        """Destoys the app. Do be careful."""
        return self.delete()

    def stream_log(self, dyno=None, lines=100, source=None, timeout=False):
        logger = self._logger(dyno=dyno, lines=lines, source=source, tail=True)

        return logger.stream(timeout=timeout)

    def get_log(self, dyno=None, lines=100, source=None, timeout=False):
        logger = self._logger(dyno=dyno, lines=lines, source=source, tail=0)

        return logger.get(timeout=timeout)

    def _logger(self, dyno=None, lines=100, source=None, tail=0):

        payload = {}
        if dyno:
            payload['dyno'] = dyno

        if tail:
            payload['tail'] = tail

        if source:
            payload['source'] = source

        if lines:
            payload['lines'] = lines

        r = self._h._http_resource(
            method='POST',
            resource=('apps', self.id, 'log-sessions'),
            data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))

        return LogSession.new_from_dict(item, h=self._h, app=self)

    def releases(self, **kwargs):
        """The releases for this app."""
        return self._h._get_resources(
            resource=('apps', self.name, 'releases'),
            obj=Release, app=self, **kwargs
        )

    def rollback(self, release):
        """Rolls back the release to the given version."""
        r = self._h._http_resource(
            method='POST',
            resource=('apps', self.name, 'releases'),
            params={'rollback': release},
            legacy=True
        )
        r.raise_for_status()
        return self.releases()[-1]


class AppTransfer(BaseResource):
    _strs = ['id', 'state']
    _map = {'app': App, 'recipient': User, 'owner': User}
    _dates = ['created_at', 'updated_at']
    _pks = ['id']

    def __init__(self):
        self.app = None
        super(AppTransfer, self).__init__()

    def __repr__(self):
        return "<apptransfer '{0}'>".format(self.id)

    def update(self, state):

        payload = {}
        payload['state'] = state

        r = self._h._http_resource(
            method='PATCH',
            resource=('account', 'app-transfers', self.id),
            data=self._h._resource_serialize(payload)
        )
        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return AppTransfer.new_from_dict(item, h=self._h, app=self)

    def delete(self):
        r = self._h._http_resource(
            method='DELETE',
            resource=('account', 'app-transfers', self.id),
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return AppTransfer.new_from_dict(item, h=self._h, app=self)


class AppFeature(BaseResource):
    _strs = ['name', 'description', 'doc_url', 'id']
    _bools = ['enabled']
    _dates = ['created_at', 'updated_at']
    _pks = ['id', 'name']

    def __init__(self):
        self.app = None
        super(AppFeature, self).__init__()

    def __repr__(self):
        return "<app_feature '{0}'>".format(self.name)

    def update(self, enabled):
        r = self._h._http_resource(
            method='POST',
            resource=('apps', self.app.id, 'features', self.id),
            data=self._h._resource_serialize({'enabled': enabled})
        )
        r.raise_for_status()
        return r.ok

    def enable(self):
        return self.update(True)

    def disable(self):
        return self.update(0)



# -*- coding: utf-8 -*-

#  /_/ _  _ _  /_      _
# / / /_'/ /_//\ /_/. /_//_/
#                    /   _/


"""
heroku3.py
~~~~~~~~~

:copyright: (c) 2013 by Heroku.
:license: MIT, see LICENSE for more details.
"""

# Meta.

__title__ = 'heroku3'
__author__ = 'Heroku'
__copyright__ = 'Copyright 2013 Heroku'
__license__ = 'MIT'

__version__ = '3.1.0'

# Module namespace.

from .core import from_key



from .  import BaseResource
import sys


if sys.version_info > (3, 0):
    from urllib.parse import quote
else:
    from urllib import quote # noqa


class Formation(BaseResource):

    _strs = ['id', 'command', 'type', 'size']
    _ints = ['quantity']
    _bools = ['attached']
    _dates = ['created_at', 'updated_at']
    _pks = ['id', 'type']

    def __init__(self):
        self.app = None
        super(Formation, self).__init__()

    def __repr__(self):
        return "<formation '{0} - {1}'>".format(self.type, self.command)

    def scale(self, quantity):
        """Scales the given process to the given number of dynos."""
        return self.update(quantity=quantity)

    def resize(self, size):
        """Resizes the Dynos to the multiple given"""
        return self.update(size=size)

    def update(self, size=None, quantity=None):

        assert(size or quantity == 0 or quantity)
        payload = {}
        if size:
            payload['size'] = size

        if quantity or quantity == 0:
            payload['quantity'] = quantity

        r = self._h._http_resource(
            method='PATCH',
            resource=('apps', self.app.id, 'formation', quote(self.type)),
            data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        return self._h._process_item(self._h._resource_deserialize(r.content.decode("utf-8")), Formation)



# -*- coding: utf-8 -*-

"""
heroku3.models
~~~~~~~~~~~~~

This module contains the models that comprise the Heroku API.
"""

from ..helpers import to_python
#from .structures import DynoListResource#, filtered_key_list_resource_factory
#from .rendezvous import Rendezvous
from pprint import pprint # noqa
import sys

if sys.version_info > (3, 0):
    from urllib.parse import quote
else:
    from urllib import quote # noqa


class BaseResource(object):

    _strs = []
    _ints = []
    _dates = []
    _bools = []
    _dicts = []
    _map = {}
    _pks = []
    order_by = 'id'

    def __init__(self):
        self._bootstrap()
        self._h = None
        super(BaseResource, self).__init__()

    def __repr__(self):
        return "<resource '{0}'>".format(self._id)

    def _bootstrap(self):
        """Bootstraps the model object based on configured values."""

        for attr in self._keys():
            setattr(self, attr, None)

    def _keys(self):
        return self._strs + self._ints + self._dates + self._bools + list(self._map.keys())

    @property
    def _id(self):
        try:
            return getattr(self, self._pks[0])
        except IndexError:
            return None

    @property
    def _ids(self):
        """The list of primary keys to validate against."""
        for pk in self._pks:
            yield getattr(self, pk)

        for pk in self._pks:

            try:
                yield str(getattr(self, pk))
            except ValueError:
                pass

    def dict(self):
        d = dict()
        for k in self.keys():
            d[k] = self.__dict__.get(k)

        return d

    def change_connection(self, h):
        self._h = h
        return self

    @classmethod
    def new_from_dict(cls, d, h=None, **kwargs):

        d = to_python(
            obj=cls(),
            in_dict=d,
            str_keys=cls._strs,
            int_keys=cls._ints,
            date_keys=cls._dates,
            bool_keys=cls._bools,
            dict_keys=cls._dicts,
            object_map=cls._map,
            _h=h
        )

        d.__dict__.update(kwargs)

        return d


class Price(BaseResource):
    """Heroku Price."""

    _strs = ['cents', 'unit']
    _pks = ['cents']

    def __init__(self):
        self.app = None
        super(Price, self).__init__()

    def __repr__(self):
        return "<price '{0} per {1}'>".format(self.cents, self.unit)


class Plan(BaseResource):
    """Heroku Addon."""

    _strs = ['id', 'name', 'description', 'state']
    _pks = ['name', 'id']
    _map = {'price': Price}
    _dates = ['created_at', 'updated_at']

    def __repr__(self):
        return "<Plan '{0}'>".format(self.name)


class Stack(BaseResource):
    """Heroku Stack."""

    _strs = ['id', 'name']
    _pks = ['id', 'name']

    def __init__(self):
        self.app = None
        super(Stack, self).__init__()

    def __repr__(self):
        return "<stack '{0}'>".format(self.name)


class User(BaseResource):
    """Heroku User."""

    _strs = ['id', 'email']
    _pks = ['id', 'email']

    def __init__(self):
        self.app = None
        super(User, self).__init__()

    def __repr__(self):
        return "<user '{0}'>".format(self.email)


#class Plan(BaseResource):
    #"""Heroku Addon."""
#
    #_strs = ['id', 'name']
    #_pks = ['id', 'name']
#
    #def __init__(self):
        #self.app = None
        #super(Plan, self).__init__()

    #def __repr__(self):
        #return "<plan '{0} {1}'>".format(self.id, self.name)
#

class RateLimit(BaseResource):
    _strs = ['remaining']
    _bools = []
    _pks = ['remaining']

    def __init__(self):
        self.app = None
        super(RateLimit, self).__init__()

    def __repr__(self):
        return "<RateLimit '{0}'>".format(self.remaining)



from .  import BaseResource
from .addon import Addon


class LogDrain(BaseResource):
    _strs = ['id', 'url']
    _map = {'addon': Addon}
    _dates = ['created_at', 'updated_at']
    _pks = ['id']

    def __init__(self):
        self.app = None
        super(LogDrain, self).__init__()

    def __repr__(self):
        return "<logdrain '{0}'>".format(self.id)

    def remove(self):
        r = self._h._http_resource(
            method='DELETE',
            resource=('apps', self.app.id, 'log-drains', self.id)
        )

        r.raise_for_status()

        return r.ok



import socket
import select
import ssl
import os
#from pprint import pprint
from urlparse import urlparse, uses_netloc
uses_netloc.append('rendezvous')


class InvalidResponseFromRendezVous(Exception):
    pass


class Rendezvous():
    def __init__(self, url, printout=False):
        self.url = url
        urlp = urlparse(url)
        self.hostname = urlp.hostname
        self.port = urlp.port
        self.secret = urlp.path[1:]
        path = os.path.dirname(os.path.realpath(__file__))
        self.cert = os.path.abspath("{0}/data/cacert.pem".format(path))
        self.data = ""
        self.printout = printout

    def start(self):

        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        # Require a certificate from the server. We used a self-signed certificate
        # so here ca_certs must be the server certificate itself.

        ssl_sock = ssl.wrap_socket(s,
                            ca_certs=self.cert,
                            cert_reqs=ssl.CERT_REQUIRED,
                            ssl_version=ssl.PROTOCOL_TLSv1)

        ssl_sock.settimeout(20)
        ssl_sock.connect((self.hostname, self.port))
        ssl_sock.write(self.secret)
        data = ssl_sock.read()
        if not data.startswith("rendezvous"):
            raise InvalidResponseFromRendezVous("The Response from the rendezvous server wasn't as expected. Response was - {0}".format(data))
        while True:
            r, w, e = select.select([ssl_sock], [], [])
            if ssl_sock in r:
                try:
                    data = ssl_sock.recv(1024)
                except ssl.SSLError as e:
                    # Ignore the SSL equivalent of EWOULDBLOCK, but re-raise other errors
                    if e.errno != ssl.SSL_ERROR_WANT_READ:
                        raise e
                    continue
                # No data means end of file
                if not data:
                    break
                if self.printout:
                    print data.rstrip('\n')
                self.data += data
        return self.data



from . import BaseResource, Plan


class Addon(BaseResource):
    """Heroku Addon."""

    _strs = ['id', 'config']
    _pks = ['id']
    _map = {'plan': Plan}
    _dates = ['created_at', 'updated_at']

    def __repr__(self):
        return "<addon '{0}'>".format(self.plan.name)

    def delete(self):
        """Uninstalls the addon"""
        r = self._h._http_resource(
            method='DELETE',
            resource=('apps', self.app.id, 'addons', self.id)
        )
        r.raise_for_status()
        return r.ok

    def upgrade(self, plan_id_or_name):

        """Upgrades an addon to the given plan."""

        payload = {'plan': plan_id_or_name}

        r = self._h._http_resource(
            method='PATCH',
            resource=('apps', self.app.id, 'addons', self.id),
            data=self._h._resource_serialize(payload)
        )

        print r.content.decode("utf-8")
        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return Addon.new_from_dict(item, h=self._h, app=self.app)



from .  import BaseResource
import requests


class LogSession(BaseResource):
    _strs = ['id', 'logplex_url', 'dyno', 'source']
    _ints = ['lines']
    _bools = ['tail']
    _dates = ['created_at', 'updated_at']
    _pks = ['id']

    def __init__(self):
        self.app = None
        super(LogSession, self).__init__()

    def __repr__(self):
        return "<logsession '{0}'>".format(self.id)

    def stream(self, timeout=False):
        r = requests.get(self.logplex_url, verify=False, stream=True, timeout=timeout)
        return r.iter_lines()

    def get(self, timeout=False):
        r = requests.get(self.logplex_url, verify=False, stream=True, timeout=timeout)
        return r.content.decode("utf-8")



from . import BaseResource
from .release import Release


class RestartRunException(Exception):
    pass


class Dyno(BaseResource):
    _strs = ['id', 'attach_url', 'size', 'command', 'name', 'state', 'type']
    _bools = ['attach']
    _ints = ['repo_size']
    _dates = ['created_at', 'updated_at']
    _map = {'release': Release}
    _pks = ['id']

    def __init__(self):
        self.app = None
        super(Dyno, self).__init__()

    def __repr__(self):
        return "<Dyno '{0} - {1}'>".format(self.name, self.command)

    def kill(self):
        r = self._h._http_resource(
            method='DELETE',
            resource=('apps', self.app.id, 'dynos', self.id)
        )

        r.raise_for_status()

        return r.ok

    def restart(self):
        if self.type == 'run':
            raise RestartRunException("Unable to restart a Process of type 'run' as it will not be respawned by Heroku")
        else:
            return self.kill()



from .  import BaseResource


class Region(BaseResource):
    _strs = ['description', 'id', 'name']
    _dates = ['created_at', 'updated_at']
    _pks = ['id', 'name']
    order_by = 'id'

    def __init__(self):
        self.app = None
        super(Region, self).__init__()

    def __repr__(self):
        return "<region '{0} {1}'>\n".format(self.name, self.id)



from . import BaseResource


class Domain(BaseResource):
    """Heroku Domain."""

    _strs = ['id', 'hostname']
    _dates = ['created_at', 'updated_at']
    _pks = ['hostname', 'id']
    order_by = 'domain'

    def __init__(self):
        self.app = None
        super(Domain, self).__init__()

    def __repr__(self):
        return "<domain '{0}'>".format(self.hostname)

    def remove(self):
        r = self._h._http_resource(
            method='DELETE',
            resource=('apps', self.app.name, 'domains', self.hostname)
        )

        r.raise_for_status()

        return r.ok



from .  import BaseResource, User


class Collaborator(BaseResource):
    """Heroku Collaborator."""

    _bools = ['silent']
    _dates = ['created_at', 'updated_at']
    _strs = ['id']
    _pks = ['id']
    _map = {'user': User}

    def __init__(self):
        self.app = None
        super(Collaborator, self).__init__()

    def __repr__(self):
        return "<collaborator '{0}'>".format(self.user.email)

    def remove(self):
        r = self._h._http_resource(
            method='DELETE',
            resource=('apps', self.app.name, 'collaborators', self.user.email)
        )
        r.raise_for_status()

        return r.ok



# coding=utf-8
import os
import heroku3
from pprint import pprint# noqa
#import socket

#import httplib
#import logging
#httplib.HTTPConnection.debuglevel = 1
#logging.basicConfig() # you need to initialize logging, otherwise you will not see anything from requests
#logging.getLogger().setLevel(logging.INFO)
#requests_log = logging.getLogger("requests.packages.urllib3")
#requests_log.setLevel(logging.INFO)
#requests_log.propagate = True

HEROKU_API_KEY = os.environ.get('HEROKU_API_KEY', False)
HEROKU_APPNAME = os.environ.get('HEROKU_APPNAME', False)
TEST_EMAIL = os.environ.get('TEST_EMAIL', False)

heroku_conn = heroku3.from_key(HEROKU_API_KEY)

#app = heroku_conn.create_app(name='testy2124app', stack_id_or_name='cedar', region_id_or_name='us')
#print app.addons()
#print heroku_conn.addons('testy123app')
#for addon in app.addons():
    #addon.delete()

#del config['TEST1']
#del config['TEST2']
#del config['TEST3']
#del config['Z01']
#del config['Z02']
#print config
#config['TEST1'] = u'MM1'
#config['TEST2'] = u'MM2'
#config['TEST3'] = u'MM3'
#config2 = heroku_conn.update_appconfig('testy123app', {u'Z01': u'A1', u'Z02': u'A2'})
#config2 = config.update({u'Z01': u'A1', u'Z02': u'A2'})
#config3 = app.config()
#print config
#print "======"
#print config2
#print "======"
#print config3
#print config['TEST1']
#print config['TEST3']

#app = heroku_conn.app('kdsjhkszdjhgksjdhfkj')
#procs = app.process_formation()
#proc = app.process_formation()['web']
#print proc.size
#print proc.quantity
#print procs
#proc.scale(0)
#app.scale_formation_process('web', 1)
#output = app.run_command('pgbackups:url')
#collab = app.add_collaborator(email=TEST_EMAIL, silent=False)
#collab = app.remove_collaborator(TEST_EMAIL)
#print newapp.collaborators()
#config = newapp.config()
#config['TEST2'] = None
#print newapp.domains()

#domain2 = newapp.add_domain('testy123.testing.com')
#print newapp.domains()
#newapp.remove_domain('testy123.testing.com')
#domain.remove()
#print newapp.domains()

#app = heroku_conn.app(HEROKU_APPNAME)
#pprint(app.addons())
#dynos = app.dynos()

#dyno = dynos['web.1']
#print dyno
#releases = app.releases(sort='asc')
#for release in releases:
    #print "{0} {1} {2} {3}".format(release.id, release.commit, release.user, release.description)
#releases = app.releases()._items.reverse()
#print releases.pop()
#print releases.pop()

#app.rollback('v108')


#apps = heroku_conn.apps(order_by='name')

#for app in apps:
    #print app.name

#app.rename('testy223')

#print app.enable_maintence_mode()
#print app.disable_maintence_mode()
#app.enable_feature('user-env-compile')
#app.disable_feature('user-env-compile')
#print app.labs()
#print heroku_conn.features()
#domain = app.add_domain('test123-1.testing.com')
#domain = app.add_domain('test123-2.testing.com')
#domain = app.add_domain('test123-3.testing.com')
#domain = app.add_domain('test123-4.testing.com')
#domain = app.add_domain('test123-5.testing.com')
#domain = app.add_domain('test123-6.testing.com')
#domain = app.add_domain('test123-7.testing.com')

#iterator = app.stream_log(lines=1)
#for line in iterator:

    # filter out keep-alive new lines
    #if line:
        #print "{0}".format(line)

#logs = app.get_log(lines=100)
#print logs

#print app.domains(limit=1)
#dyno = app.run_command('fab -l', printout=True)
#dyno.remove()
#proc = heroku_conn.apps()['testy123app'].process_formation()['web']
#print proc.size
#print proc.quantity

#formations = app.process_formation()
#print formations['web']
#for formation in formations:
    #formation.resize(1)
    #print app._h._last_request_id

#print app.dynos()['web.1']
#print dynos['web.1']

#print heroku_conn.apps()['testy123app']
#print heroku_conn.apps()['d32b74d8-f5cf-4e3e-95dd-a601668fdb0c']
#for dyno in app.dynos():
    #print dyno
    #print dyno.command
    #dyno.restart()

#app.restart()
#del config['TEST2']

#newapp.remove_collaborator('testing125146513@mail.com')
#collab.remove()
#pprint(newapp.addons)
#app = heroku_conn.app('testy123app')
#for addon in app.addons:
    #print addon.app.name, " - ", addon.plan.name

#addons = heroku_conn.addon_services()
#pprint(addons)

#pg_addon = heroku_conn.addon_services('6235c964-8b3c-47e0-952f-8d8f6a2d53f5')
#pg_addon = heroku_conn.addon_services(id_or_name='heroku-postgresql')
#pprint(pg_addon)


#for addon in addons:
    #print addon.name, " - ", addon.id, " - ", addon.id, " - ", addon.price
    #addon.upgrade(plan_id_or_name='heroku-postgresql:basic')
    #addon.delete()

#app.delete()

print heroku_conn._last_request_id



from .. import BaseResource


class AccountFeature(BaseResource):
    _strs = ['name', 'description', 'doc_url', 'id']
    _bools = ['enabled']
    _dates = ['created_at', 'updated_at']
    _pks = ['id', 'name']

    def __init__(self):
        self.app = None
        super(AccountFeature, self).__init__()

    def __repr__(self):
        return "<account_feature '{0}'>".format(self.name)

    def update(self, enabled):
        r = self._h._http_resource(
            method='POST',
            resource=('account', 'features', self.id),
            data=self._h._resource_serialize({'enabled': enabled})
        )
        r.raise_for_status()
        return r.ok

    def enable(self):
        return self.update(True)

    def disable(self):
        return self.update(0)



# -*- coding: utf-8 -*-

"""
heroku3.helpers
~~~~~~~~~~~~~~

This module contians the helpers.
"""

from datetime import datetime

from dateutil.parser import parse as parse_datetime

import sys

if sys.version_info > (3, 0):
    basestring = (str, bytes)


def is_collection(obj):
    """Tests if an object is a collection."""

    col = getattr(obj, '__getitem__', False)
    val = False if (not col) else True

    if isinstance(obj, basestring):
        val = False

    return val


# from kennethreitz/python-github3
def to_python(obj,
    in_dict,
    str_keys=None,
    date_keys=None,
    int_keys=None,
    object_map=None,
    bool_keys=None,
    dict_keys=None,
    **kwargs):
    """Extends a given object for API Consumption.

    :param obj: Object to extend.
    :param in_dict: Dict to extract data from.
    :param string_keys: List of in_dict keys that will be extracted as strings.
    :param date_keys: List of in_dict keys that will be extrad as datetimes.
    :param object_map: Dict of {key, obj} map, for nested object results.
    """

    d = dict()

    if str_keys:
        for in_key in str_keys:
            d[in_key] = in_dict.get(in_key)

    if date_keys:
        for in_key in date_keys:
            in_date = in_dict.get(in_key)
            if in_date is not None:
                try:
                    out_date = parse_datetime(in_date)
                except TypeError as e:
                    raise e
                    out_date = None

                d[in_key] = out_date
            else:
                d[in_key] = None

    if int_keys:
        for in_key in int_keys:
            if (in_dict is not None) and (in_dict.get(in_key) is not None):
                d[in_key] = int(in_dict.get(in_key))

    if bool_keys:
        for in_key in bool_keys:
            if in_dict.get(in_key) is not None:
                d[in_key] = bool(in_dict.get(in_key))

    if dict_keys:
        for in_key in dict_keys:
            if in_dict.get(in_key) is not None:
                d[in_key] = dict(in_dict.get(in_key))

    if object_map:
        for (k, v) in object_map.items():
            if in_dict.get(k):
                d[k] = v.new_from_dict(in_dict.get(k))

    obj.__dict__.update(d)
    obj.__dict__.update(kwargs)

    # Save the dictionary, for write comparisons.
    # obj._cache = d
    # obj.__cache = in_dict

    return obj


# from kennethreitz/python-github3
def to_api(in_dict, int_keys=None, date_keys=None, bool_keys=None):
    """Extends a given object for API Production."""

    # Cast all int_keys to int()
    if int_keys:
        for in_key in int_keys:
            if (in_key in in_dict) and (in_dict.get(in_key, None) is not None):
                in_dict[in_key] = int(in_dict[in_key])

    # Cast all date_keys to datetime.isoformat
    if date_keys:
        for in_key in date_keys:
            if (in_key in in_dict) and (in_dict.get(in_key, None) is not None):

                _from = in_dict[in_key]

                if isinstance(_from, basestring):
                    dtime = parse_datetime(_from)

                elif isinstance(_from, datetime):
                    dtime = _from

                in_dict[in_key] = dtime.isoformat()

            elif (in_key in in_dict) and in_dict.get(in_key, None) is None:
                del in_dict[in_key]

    # Remove all Nones
    for k, v in in_dict.items():
        if v is None:
            del in_dict[k]

    return in_dict



from ...models import BaseResource
from ..key import Key
from .feature import AccountFeature
import sys

if sys.version_info > (3, 0):
    from urllib.parse import quote
else:
    from urllib import quote # noqa


class Account(BaseResource):

    _strs = ['email', 'id']
    _bools = ['allow_tracking', 'beta', 'verified']
    _pks = ['id']
    _dates = ['created_at', 'last_login', 'updated_at']

    def __repr__(self):
        return "<account '{0}'>".format(self.email)

    def keys(self, **kwargs):
        """The collaborators for this app."""
        return self._h._get_resources(
            resource=('account', 'keys'),
            obj=Key, app=self, **kwargs
        )

    def add_key(self, key):
        r = self._h._http_resource(
            method='POST',
            resource=('account', 'keys'),
            data=self._h._resource_serialize({'public_key': key})
        )
        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return Key.new_from_dict(item, h=self._h)

    def remove_key(self, key_or_fingerprint):
        """Deletes the key."""
        r = self._h._http_resource(
            method='DELETE',
            resource=('account', 'keys', quote(key_or_fingerprint))
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return Key.new_from_dict(item, h=self._h)

    def disable_feature(self, id_or_name):
        return self.update_feature(id_or_name, 0)

    def enable_feature(self, id_or_name):
        return self.update_feature(id_or_name, True)

    def update_feature(self, id_or_name, enabled):

        payload = {'enabled': enabled}
        r = self._h._http_resource(
            method='PATCH',
            resource=('account', 'features', id_or_name),
            data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return AccountFeature.new_from_dict(item, h=self._h, app=self)

    def change_password(self, current_password, new_password):
        r = self._h._http_resource(
            method='PUT',
            resource=('account', 'password'),
            data=self._h._resource_serialize({'current_password': current_password, 'new_password': new_password})
        )
        r.raise_for_status()
        return r.ok



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup # NOQA


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

required = [
    'requests>=1.2.3',
    'simplejson==3.3.1',
    'python-dateutil==1.5'
]


setup(
    name='heroku3',
    version='3.1.1',
    description='Heroku API Wrapper.',
    long_description=open('README.rst').read() + '\n\n' +
                     open('HISTORY.rst').read(),
    author='Martin Moss',
    author_email='martin_moss@btinternet.com',
    url='https://github.com/martyzz1/heroku3.py',
    download_url='https://github.com/martyzz1/heroku3.py/tarball/v3.1.1',
    packages=['heroku3'],
    package_data={'': ['LICENSE', ]},
    include_package_data=True,
    install_requires=required,
    license='MIT',
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        # 'Programming Language :: Python :: 3.0',
        # 'Programming Language :: Python :: 3.1',
    ),
)



import json


class ConfigVars(object):
    """Heroku ConfigVars."""

    def __init__(self):

        self.data = {}
        self.app = None
        self._h = None

        super(ConfigVars, self).__init__()

    def __repr__(self):
        return repr(self.data)

    def __contains__(self, key):
        v = None
        try:
            v = self[key]
        except KeyError:
            return False
        else:
            if v is None:
                return False
            else:
                return True

    def __getitem__(self, key):
        return self.data.get(key)

    def __setitem__(self, key, value):
        # API expects JSON.
        payload = json.dumps({key: value})

        r = self._h._http_resource(
            method='PATCH',
            resource=('apps', self.app.name, 'config-vars'),
            data=payload
        )

        self.data[key] = value
        return r.ok

    def __delitem__(self, key):
        data = self._h._resource_serialize({key: None})
        r = self._h._http_resource(
            method='PATCH',
            resource=('apps', self.app.name, 'config-vars'),
            data=data
        )

        del self.data[key]
        return r.ok

    def update(self, newconf):
        payload = self._h._resource_serialize(newconf)
        r = self._h._http_resource(
            method='PATCH',
            resource=('apps', self.app.name, 'config-vars'),
            data=payload
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return ConfigVars.new_from_dict(item, h=self)

    def to_dict(self):
        return self.data

    @classmethod
    def new_from_dict(cls, d, h=None, **kwargs):
        # Override normal operation because of crazy api.
        c = cls()
        c.data = d
        c._h = h
        c.app = kwargs.get('app')

        return c



from .  import BaseResource


class Key(BaseResource):
    """Heroku SSH Key."""

    _strs = ['id', 'public_key', 'email', 'fingerprint']
    _dates = ['created_at', 'updated_at']
    _pks = ['id']

    def __init__(self):
        super(Key, self).__init__()

    def __repr__(self):
        return "<key '{0}-{1}'>".format(self.id, self.email)

    def delete(self):
        """Deletes the key."""
        r = self._h._http_resource(
            method='DELETE',
            resource=('account', 'keys', self.id)
        )

        r.raise_for_status()
        return self



from . import BaseResource, User


class Grant(BaseResource):
    """Heroku Grant"""

    _strs = ['code', 'id', 'type']
    _ints = ['expires_in']
    _pks = ['id']

    def __init__(self):
        #self.app = None
        super(Grant, self).__init__()

    def __repr__(self):
        return "<grant '{0} {1} {2}'>".format(self.id, self.code, self.expires_in)


class RefreshToken(BaseResource):
    """Heroku RefreshToken"""

    _strs = ['token', 'id']
    _ints = ['expires_in']
    _pks = ['id']

    def __init__(self):
        #self.app = None
        super(RefreshToken, self).__init__()

    def __repr__(self):
        return "<refreshtoken '{0} {1} {2}'>".format(self.id, self.token, self.expires_in)


class AccessToken(BaseResource):
    """Heroku AccessToken"""

    _strs = ['token', 'id']
    _ints = ['expires_in']
    _pks = ['id']

    def __init__(self):
        #self.app = None
        super(AccessToken, self).__init__()

    def __repr__(self):
        return "<AccessToken '{0} {1} {2}'>".format(self.id, self.token, self.expires_in)


#class Client(BaseResource):
    #"""Heroku Client"""
#
    #_strs = ['id', 'name', 'redirect_uri']
    #_pks = ['id']
#
    #def __init__(self):
        #self.app = None
        #super(Client, self).__init__()
#
    #def __repr__(self):
        #return "<Client '{0} {1} {2}'>".format(self.id, self.name, self.redirect_uri)


class OAuthClient(BaseResource):
    """Heroku OAuthClient"""

    _strs = ['id', 'name', 'redirect_uri', 'secret']
    _dates = ['created_at', 'updated_at']
    _bools = ['ignores_delinquent']
    _pks = ['id']

    def __init__(self):
        #self.app = None
        super(OAuthClient, self).__init__()

    def __repr__(self):
        return "<OAuthClient '{0} {1} {2} {3}'>".format(self.id, self.name, self.redirect_uri, self.created_at)

    def delete(self):
        """
        Destroys the current OAuthClient
        """
        r = self._h._http_resource(
            method='DELETE',
            resource=('oauth', 'clients', self.id)
        )
        r.raise_for_status()
        return r.ok

    def update(self, name=None, redirect_uri=None):

        assert (name or redirect_uri)
        payload = {'name': name, 'redirect_uri': redirect_uri}
        r = self._h._http_resource(
            method='PATCH',
            resource=('oauth', 'clients', self.id),
            data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return OAuthClient.new_from_dict(item, h=self._h)


class OAuthAuthorization(BaseResource):
    _strs = ['id']
    _dates = ['created_at', 'updated_at']
    _map = {'grant': Grant, 'refresh_token': RefreshToken, 'access_token': AccessToken, 'client': OAuthClient}
    _pks = ['id']
    order_by = 'id'

    def __init__(self):
        #self.app = None
        super(OAuthAuthorization, self).__init__()

    def __repr__(self):
        return "<OAuthAuthorization '{0} {1} {2}'>\n".format(self.id, self.created_at, self.client.id)

    def delete(self):
        """
        Destroys the current OAuthAuthorization
        """
        r = self._h._http_resource(
            method='DELETE',
            resource=('oauth', 'authorizations', self.id)
        )
        r.raise_for_status()
        return r.ok


class OAuthToken(BaseResource):
    _strs = ['id']
    _dates = ['created_at', 'updated_at']
    _map = {
                'grant': Grant,
                'refresh_token': RefreshToken,
                'access_token': AccessToken,
                'client': OAuthClient,
                'authorization': OAuthAuthorization,
                'user': User
        }
    _pks = ['id']
    order_by = 'id'

    def __init__(self):
        #self.app = None
        super(OAuthToken, self).__init__()

    def __repr__(self):
        return "<OAuthToken '{0} {1} {2} {3}'>\n".format(self.id, self.created_at, self.user.id, self.grant.type)


class Session(BaseResource):
    _strs = ['id']
    _pks = ['id']
    order_by = 'id'

    def __init__(self):
        #self.app = None
        super(Session, self).__init__()

    def __repr__(self):
        return "<Session '{0}'>\n".format(self.id)
